<!doctype html>
<html>
<head>
    <meta charset="utf-8" />
    <title>Welcome to Ursa2</title>
    <style type="text/css" media="screen">
            *{ margin:0; padding:0; }
            body{ background:#333; color:#fff; font-size:14px; line-height:1.3; font-family:sans-serif; }
            .wrapper{ width:95%; margin: 20px auto; }
            h1{ font-size:20px; }
            table{ width: 100%; }
            caption { font-size: 25px; height: 45px; line-height: 45px; }
            th,td{ text-align: left; vertical-align: top; }
            thead{border-bottom: 1px dashed #efefef;}
            .line{ width: 100%; }
            ul{ list-style: none; }
            .tlist li{ margin:10px 0; }
            .tlist a{ color:#fff; text-decoration:none; border-bottom:1px #efefef dotted; }
            .tlist .title{ margin-right:20px; font-size:16px; }
            .tlist .mgr{font-size:11px;}
            footer{text-align: center;border-top: 1px solid #222;}
        </style>
</head>
<body>
    <div class="wrapper">
        <table>
            <caption>模板列表</caption>
            <thead>
                <tr>
                    <th>入口(main)</th>
                    <th>模块(module)</th>
                    <th>通用(common)</th>
                </tr>
            </thead>
            <tbody>
                <tr>
                    <td>
                        <ul class="tlist">
                            {% for tpl in tpls %}
                            <li>
                                <a class="title" href="{{tpl}}.ut">{{tpl}}</a>
                                <a class="mgr" href="{{tpl}}.m">mgr</a>
                                <a class="mgr" href="{{tpl}}.data">data</a>
                            </li>
                            {% endfor %}
                        </ul>
                    </td>
                    <td>
                        <ul class="tlist">
                            {% for tpl in module_tpls %}
                            <li>
                                <a class="title" href="{{tpl}}.ut">{{tpl}}</a>
                                <a class="mgr" href="{{tpl}}.m">mgr</a>
                                <a class="mgr" href="{{tpl}}.data">data</a>
                            </li>
                            {% endfor %}
                        </ul>
                    </td>
                    <td>
                        <ul class="tlist">
                            {% for tpl in common_tpls %}
                            <li>
                                <a class="title" href="{{tpl}}.ut">{{tpl}}</a>
                                <a class="mgr" href="{{tpl}}.m">mgr</a>
                                <a class="mgr" href="{{tpl}}.data">data</a>
                            </li>
                            {% endfor %}
                        </ul>
                    </td>
                </tr>
            </tbody>
        </table>
    </div>
    <footer>&copy;ufo</footer>
</body>
</html>